@@ -49,16 +49,26 @@ my $changes     = CPAN::Changes->new();
 my $changes_all = CPAN::Changes->new();
 my $changes_dev = CPAN::Changes->new();
 
-my $master_changes = CPAN::Changes->load_string( path('./Changes')->slurp_utf8 );
+my $master_changes = CPAN::Changes->load_string( path('./Changes')->slurp_utf8, next_token => qr/{{\$NEXT}}/ );
 
 while ( @tags > 2 ) {
   my ( $old, $new ) = ( $tags[-2], $tags[-1] );
   pop @tags;
 
   my $date;
-  if ( my $master_release = $master_changes->release($new) ) {
+  my $master_release;
+  if ( $master_release = $master_changes->release($new) ) {
     $date = $master_release->date();
   }
+  else {
+    print "$new not on master Changelog\n";
+    if ( $new eq 'build/master' ) {
+      $master_release = [ $master_changes->releases ]->[-1];
+      print " ... using " . $master_release->version . " instead \n";
+
+      #('{{$NEXT}}');
+    }
+  }
   my $version = $new;
   if ( $new eq 'build/master' ) {
     $version = $build_master_version;
@@ -77,24 +87,66 @@ while ( @tags > 2 ) {
   next unless defined $old_meta_sha1 and length $old_meta_sha1;
   next unless defined $new_meta_sha1 and length $new_meta_sha1;
 
-  open my $old_meta, '-|', 'git', 'cat-file', '-p', $old_meta_sha1;
-  open my $new_meta, '-|', 'git', 'cat-file', '-p', $new_meta_sha1;
-
   my $diff = depsdiff->new(
     json_a => ( join qq[\n], $git->cat_file( '-p', $old_meta_sha1 ) ),
     json_b => ( join qq[\n], $git->cat_file( '-p', $new_meta_sha1 ) ),
   );
   $diff->execute;
 
-  #print "\e[31m$new - \e[0m\n";
+  $master_release->delete_group('Dependencies::Stats') if $master_release;
 
+  next unless keys %{ $diff->cache };
+
+  if ($master_release) {
+    my $phases = {};
+    for my $key ( sort keys %{ $diff->cache } ) {
+      if ( $key =~ qr{^Dependencies::(\S+)\s+/\s+(\S+)\s+(\S+)$}msx ) {
+        my ( $dir, $phase, $rel ) = ( $1, $2, $3 );
+        my $phase_m = "$phase";
+        $phases->{$phase_m} = { Added => 0, Upgrade => 0, Downgrade => 0, Removed => 0 }
+          unless exists $phases->{$phase_m};
+        for my $entry ( @{ $diff->cache->{$key} } ) {
+          if ( $dir eq 'Added' or $dir eq 'Removed' ) {
+            $phases->{$phase_m}->{$dir}++;
+          }
+          else {
+            if ( $entry =~ /(\S+)\s+→\s+(\S+)/ ) {
+              my ( $lhs, $rhs ) = ( $1, $2 );
+              require version;
+              my $lhs_v = version->parse($lhs);
+              my $rhs_v = version->parse($rhs);
+              if ( $lhs_v < $rhs_v ) {
+                $phases->{$phase_m}->{Upgrade}++;
+              }
+              else {
+                $phases->{$phase_m}->{Downgrade}++;
+              }
+            }
+          }
+        }
+      }
+    }
+    my @changes = ();
+    for my $phase ( sort keys %{$phases} ) {
+      my @parts;
+      if ( $phases->{$phase}->{Added} > 0 ) {
+        push @parts, "+" . $phases->{$phase}->{Added};
+      }
+      if ( $phases->{$phase}->{Upgrade} > 0 ) {
+        push @parts, "↑" . $phases->{$phase}->{Upgrade};
+      }
+      if ( $phases->{$phase}->{Downgrade} > 0 ) {
+        push @parts, "↓" . $phases->{$phase}->{Downgrade};
+      }
+      if ( $phases->{$phase}->{Removed} > 0 ) {
+        push @parts, "-" . $phases->{$phase}->{Removed};
+      }
+      push @changes, $phase . ': ' . ( join q[ ], @parts );
+    }
+    $master_release->add_changes( { group => 'Dependencies::Stats' },
+      'Dependencies changed, see Changes.deps{,.all,.dev} for details', @changes );
+  }
   for my $key ( sort keys %{ $diff->cache } ) {
-
-    #print "\t\e[32m$key\e[0m\n";
-    #for my $value ( @{ $diff->cache->{$key} } ) {
-    #    binmode *STDOUT, ':utf8';
-    #    print "\t\t$value\n";
-    #}
     my $label = $key;
     $label =~ s/Dependencies:://msx;
     $changes_all->release($version)->add_changes( { group => $label }, @{ $diff->cache->{$key} } );
@@ -109,4 +161,4 @@ while ( @tags > 2 ) {
 path('./Changes.deps.all')->spew_utf8( $changes_all->serialize );
 path('./Changes.deps')->spew_utf8( $changes->serialize );
 path('./Changes.deps.dev')->spew_utf8( $changes_dev->serialize );
-
+path('./Changes')->spew_utf8( $master_changes->serialize );
